import logging

from cifsdk.client.http.base import Base
from cifsdk.client import Client
from csirtg_indicator import Indicator

from .utils import _check_data, _check_status
from pprint import pprint

from .constants import RETRIES_DELAY, TRACE, TIMEOUT, RETRIES

logger = logging.getLogger(__name__)


class HTTP(Client, Base):

    def indicators_search(self, filters):
        if isinstance(filters, list):
            return self._post('indicators/bulk', filters, 200)

        data = self._get('indicators', params=filters)
        return data

    def indicators_search_bulk(self, data):
        return self._post('indicators/bulk', data, 200)

    def indicators_create(self, data):
        if isinstance(data, Indicator):
            data = data.__dict__()

        if type(data) == dict:
            data = [data]

        if isinstance(data, list) and isinstance(data[0], Indicator):
            data = [i.__dict__() for i in data]

        rv = self._put('indicators', data)
        return rv["data"]

    def indicators_delete(self, filters):
        rv = self._delete('indicators', params=filters)
        return rv["data"]



#!/usr/bin/env python3

import logging
import sys
import textwrap
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

from csirtg_indicator.format import FORMATS
from cifsdk.utils import setup_logging, get_argument_parser
from csirtg_indicator import Indicator
from cifsdk.client.http import HTTP as Client

from cifsdk.constants import REMOTE, SEARCH_LIMIT, FORMAT, COLUMNS, EXPERT, \
    PROFILES, VALID_FILTERS

from pprint import pprint

logger = logging.getLogger(__name__)


def _check_filters(filters):
    if not isinstance(filters, dict):
        return

    if EXPERT is True:
        return

    if filters.get('indicator'):
        return

    if not filters.get('itype'):
        print('\nmissing --itype\n\n')
        raise SystemExit

    if not filters.get('indicator') and not filters.get('tags'):
        print('\nmissing --tags '
              '[phishing|malware|botnet|scanner|pdns|whitelist|...]\n\n')
        raise SystemExit


def _search(cli, args, options, filters):

    if not args.profile:
        _check_filters(filters)

    fmt = options.get('format')
    if args.profile:
        for k, v in PROFILES[args.profile].items():
            if k == 'format':
                fmt = v
            else:
                filters[k] = v

    try:
        rv = cli.search(filters)

    except PermissionError:
        logger.error('unauthorized')

    except ConnectionRefusedError:
        print("\ncif-router is either too busy or requires a restart...")
        print("if the problem continues, try increasing the amount of memory "
              "and cpus for the system")
        print("\nhttps://github.com/csirtgadgets/cif-v5/wiki/FAQ\n"
              "https://csirtg.io/support\n")

        raise SystemExit

    except KeyboardInterrupt:
        pass

    except Exception as e:
        import traceback
        traceback.print_exc()
        logger.error(e, exc_info=True)

    else:
        for l in FORMATS[fmt](data=rv):
            print(l.rstrip("\n"))

    raise SystemExit


def _create(cli, args, options):
    print("submitting {0}".format(options.get("submit")))
    i = Indicator(indicator=args.indicator, tags=args.tags,
                  confidence=args.confidence)

    rv = cli.indicators_create(i)

    print('success id: {}\n'.format(rv))


def _delete(cli, args, filters):
    if args.id:
        filters = {'id': args.id}

    filters = {f: filters[f] for f in filters if filters.get(f)}
    print("deleting {0}".format(filters))
    rv = cli.indicators_delete(filters)

    print('deleted: {}'.format(rv))


def main():  # pragma: no cover
    p = get_argument_parser()
    p = ArgumentParser(
        description=textwrap.dedent('''\
        example usage:
            $ cif -q example.org -d
            $ cif --search 1.2.3.0/24
            $ cif --profile zeek
        '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='cif',
        parents=[p]
    )
    p.add_argument('--remote', help='specify API remote [default %(default)s]',
                   default=REMOTE)
    p.add_argument('--no-verify-ssl', action='store_true')

    p.add_argument("--create", action="store_true", help="create an indicator")
    p.add_argument('--delete', action='store_true')
    p.add_argument('-q', '--search', help="search")

    p.add_argument('--itype', help='filter by indicator type')
    p.add_argument('--reported_at', help='specify reported_at filter')

    p.add_argument('-n', '--nolog', help='do not log search',
                   action='store_true')

    p.add_argument('-f', '--format',
                   help='specify output format [default: %(default)s]"',
                   default=FORMAT, choices=FORMATS.keys())

    p.add_argument('--indicator', help='indicator (ip, url, etc..) '
                                       'to search for')
    p.add_argument('--confidence', help="specify confidence level")
    p.add_argument('--tags', nargs='+', help='filter by tag(s)')
    p.add_argument('--provider', help='provider to filter by')
    p.add_argument('--asn', help='filter by asn')
    p.add_argument('--cc', help='filter by country code')
    p.add_argument('--asn-name', help='filter by asn name')
    p.add_argument('--rdata', help='filter by rdata')
    p.add_argument('--groups', help='filter by group(s)')

    p.add_argument('--days', help='filter results within last X days')
    p.add_argument('--today', action='store_true',
                   help='auto-sets reported_at to today, 00:00:00Z (UTC)')

    p.add_argument('--limit', help='limit results [default %(default)s]',
                   default=SEARCH_LIMIT)

    p.add_argument('--columns',  default=','.join(COLUMNS),
                   help='specify output columns [default %(default)s]')

    p.add_argument('--no-feed', action='store_true',
                   help='return a non-filtered dataset (no whitelist applied)')

    p.add_argument('--profile', help='specify feed profile',
                   choices=PROFILES.keys())

    args = p.parse_args()

    setup_logging(args)

    opts = vars(args)

    options = {}
    for k, v in opts.items():
        if v:
            options[k] = v

    if args.remote.startswith('http'):
        verify_ssl = True
        if args.no_verify_ssl:
            verify_ssl = False

        if args.remote == 'https://localhost':
            verify_ssl = False

        cli = Client(verify_ssl=verify_ssl)

    else:
        from cifsdk.client.zeromq import ZMQ
        cli = ZMQ()

    filters = {e: options.get(e) for e in VALID_FILTERS}

    if args.search:
        filters['indicator'] = args.search

    for k, v in filters.items():
        if v is True:
            filters[k] = 1

        if v is False:
            filters[k] = 0

    if options.get("create"):
        _create(cli, args, filters)
        raise SystemExit

    if options.get("delete"):
        _delete(cli, args, filters)
        raise SystemExit

    if not sys.stdin.isatty():
        buffer = sys.stdin.read().rstrip("\n").split("\n")

        filters = [{'indicator': i} for i in buffer]

    _search(cli, args, options, filters)


if __name__ == "__main__":
    main()



import logging
import requests
import json
from time import sleep
import random

from cifsdk.constants import VERSION, REMOTE

from cifsdk.client.http.utils import _check_data, _check_status
from pprint import pprint

from .constants import RETRIES_DELAY, TRACE, TIMEOUT, RETRIES

s, e = RETRIES_DELAY.split(',')
RETRIES_DELAY = random.uniform(int(s), int(e))

requests.packages.urllib3.disable_warnings()

logger = logging.getLogger(__name__)

logger.setLevel(logging.WARNING)
logging.getLogger('requests.packages.urllib3.connectionpool')\
    .setLevel(logging.ERROR)

if TRACE:
    logger.setLevel(logging.DEBUG)
    logging.getLogger('requests.packages.urllib3.connectionpool')\
        .setLevel(logging.DEBUG)


class Base(object):

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return self

    def __init__(self, timeout=int(TIMEOUT), **kwargs):
        self.proxy = kwargs.get('proxy')
        self.verify_ssl = kwargs.get('verify_ssl', True)
        self.nowait = kwargs.get('nowait', False)
        self.timeout = timeout

        self.remote = kwargs.get('remote', REMOTE)

        self.session = requests.Session()
        self.session.headers["Accept"] = 'application/vnd.cif.v5+json'
        self.session.headers['User-Agent'] = 'cifsdk-py/{}'.format(VERSION)
        self.session.headers['Accept-Encoding'] = 'deflate'

    def __get(self, uri, params):
        resp = self.session.get(uri, params=params, verify=self.verify_ssl,
                                timeout=self.timeout)
        logger.debug(resp.text)
        n = RETRIES
        try:
            _check_status(resp, expect=200)
            n = 0
            return resp

        except Exception as e:
            if resp.status_code == 429 or resp.status_code in [503, 504]:
                logger.error(e)
            else:
                raise e

        for nn in (1, n):
            logger.warning('setting random retry interval..')
            logger.warning('retrying in %.00fs' % RETRIES_DELAY)
            sleep(RETRIES_DELAY)

            resp = self.session.get(uri, params=params, verify=self.verify_ssl,
                                    timeout=self.timeout)
            if resp.status_code == 200:
                break

            if nn == n:
                raise ConnectionAbortedError('system seems busy.. try again '
                                             'later')

        return resp

    def _get(self, uri, params={}, retry=True):
        if not uri.startswith('http'):
            uri = "%s/%s" % (self.remote, uri)

        resp = self.__get(uri, params)

        data = resp.content

        ss = (int(resp.headers['Content-Length']) / 1024 / 1024)
        logger.info('processing %.2f megs' % ss)

        return _check_data(json.loads(data.decode('utf-8')))

    def _post(self, uri, data, expect=201):
        if not uri.startswith('http'):
            uri = "%s/%s" % (self.remote, uri)

        data = json.dumps(data)

        if self.nowait:
            uri = '{}?nowait=1'.format(uri)

        if isinstance(data, str):
            data = data.encode('utf-8')

        headers = {'Content-Type': 'application/json'}
        logger.debug('submitting')
        resp = self.session.post(uri, data=data, verify=self.verify_ssl,
                                 headers=headers, timeout=self.timeout)
        logger.debug(resp.content)
        logger.debug(resp.status_code)
        n = RETRIES
        try:
            _check_status(resp, expect=expect)
            n = 0

        except Exception as e:
            if resp.status_code == 429 or resp.status_code in [503, 504]:
                logger.error(e)
            else:
                raise e

        while n != 0:
            logger.info('setting random retry interval to spread out the load')
            logger.info('retrying in %.00fs' % RETRIES_DELAY)
            sleep(RETRIES_DELAY)

            resp = self.session.post(uri, data=data, verify=self.verify_ssl,
                                     headers=headers, timeout=self.timeout)

            if resp.status_code in [200, 201]:
                break

            if n == 0:
                logger.warning('system seems busy.. try again later')
                raise ConnectionRefusedError

        return json.loads(resp.content.decode('utf-8'))

    def _delete(self, uri, params={}):
        if not uri.startswith('http'):
            uri = "%s/%s" % (self.remote, uri)

        params = {f: params[f] for f in params if params.get(f)}

        for f in ['nolog', 'limit']:
            if params.get(f):
                del params[f]

        resp = self.session.delete(uri, data=json.dumps(params),
                                   verify=self.verify_ssl,
                                   timeout=self.timeout)
        _check_status(resp)
        return json.loads(resp.content.decode('utf-8'))

    def _put(self, uri, data):
        if not uri.startswith('http'):
            uri = "%s/%s" % (self.remote, uri)

        resp = self.session.put(uri, data=json.dumps(data))
        _check_status(resp)
        return json.loads(resp.content)



import os


SNDTIMEO = os.getenv('ZMQ_SNDTIMEO', '30000')
SNDTIMEO = int(SNDTIMEO)
RCVTIMEO = os.getenv('ZMQ_RCVTIMEO', '30000')
RCVTIMEO = int(RCVTIMEO)

LINGER = 3
ENCODING_DEFAULT = "utf-8"

SEARCH_LIMIT = 100
FIREBALL_SIZE = os.getenv('CIFSDK_CLIENT_ZMQ_FIREBALL_SIZE', 500)
FIREBALL_SIZE = int(FIREBALL_SIZE)
TRACE = os.getenv('CIFSDK_CLIENT_ZMQ_TRACE')



import pkgutil
import logging
from argparse import ArgumentParser
import signal
import os
import yaml
from importlib import import_module
import time

from cifsdk.constants import LOG_FORMAT, RUNTIME_PATH, LOGLEVEL, VERSION
from . import color


def settings(f):
    if not os.path.exists(f):
        return

    with open(f, "r") as f:
        return yaml.load(f, Loader=yaml.SafeLoader)


def get_argument_parser():
    BasicArgs = ArgumentParser(add_help=False)
    BasicArgs.add_argument('-d', '--debug', dest='debug', action="store_true")
    BasicArgs.add_argument('-v', '--verbose', action='store_true')
    BasicArgs.add_argument('-V', '--version', action='version',
                           version=VERSION)
    BasicArgs.add_argument(
        "--runtime-path",
        help="specify the runtime path [default %(default)s]",
        default=RUNTIME_PATH
    )
    return ArgumentParser(parents=[BasicArgs], add_help=False)


def load_plugin(path, plugin, prefix='cif_'):
    for loader, modname, is_pkg in pkgutil.iter_modules(path):
        if modname == plugin or modname == 'z{}'.format(plugin):
            p = loader.find_module(modname).load_module(modname)
            p = p.Plugin
            return p

    p = import_module(f"{prefix}{plugin}")
    return p.Plugin


def load_plugins(path):
    p = []
    for loader, modname, is_pkg in pkgutil.iter_modules(path):
        p.append(loader.find_module(modname).load_module(modname))

    return p


def init_plugins(path):
    return {
        name: import_module(name)
        for finder, name, ispkg
        in pkgutil.iter_modules()
        if name.startswith(path)
    }


def setup_logging(args):
    loglevel = logging.getLevelName(LOGLEVEL)

    if args.debug:
        loglevel = logging.DEBUG

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)


def setup_signals(name):
    logger = logging.getLogger(__name__)

    def sigterm_handler(_signo, _stack_frame):
        logger.info('SIGTERM Caught for {}, shutting down...'.format(name))
        raise SystemExit

    signal.signal(signal.SIGTERM, sigterm_handler)


def setup_runtime_path(path):
    if not os.path.isdir(path):
        os.mkdir(path)


class Timer:
    def __enter__(self):
        self.start = time.clock()
        return self

    def __exit__(self, *args):
        self.end = time.clock()
        self.interval = self.end - self.start



from base64 import b64decode
import json

basestring = (str, bytes)


def _check_data(msgs):
    if isinstance(msgs, list):
        return msgs

    if msgs.get('status', False) not in ['success', 'failure']:
        raise RuntimeError(msgs)

    if msgs.get('status') == 'failure':
        raise TypeError(msgs['message'])

    if msgs['data'] == '{}':
        msgs['data'] = []

    # check to see if it's straight elasticsearch json
    if isinstance(msgs['data'], basestring) and msgs['data']\
            .startswith('{"hits":{"hits":[{"_source":'):
        msgs['data'] = json.loads(msgs['data'])
        msgs['data'] = [r['_source'] for r in msgs['data']['hits']['hits']]

    if not isinstance(msgs['data'], list):
        msgs['data'] = [msgs['data']]

    for m in msgs['data']:
        if not isinstance(m, dict):
            continue

        if not m.get('message'):
            continue

        try:
            m['message'] = b64decode(m['message'])
        except Exception as e:
            pass

    return msgs


def _check_status(resp, expect=200):
    if resp.status_code == expect:
        return

    if resp.status_code == 400:
        r = json.loads(resp.text)
        raise TypeError(r['message'])

    if resp.status_code == 401:
        raise PermissionError('unauthorized')

    if resp.status_code == 404:
        raise LookupError('not found')

    if resp.status_code == 408:
        raise TimeoutError('timeout')

    if resp.status_code == 422:
        msg = json.loads(resp.text)
        raise ValueError(msg['message'])

    if resp.status_code == 429:
        raise ConnectionRefusedError('RateLimit exceeded')

    if resp.status_code in [503, 504]:
        raise ConnectionAbortedError('system seems busy..')

    if resp.status_code == 500:
        raise ConnectionError('System Error, Try again later..')

    if resp.status_code != expect:
        msg = 'unknown: %s' % resp.content
        raise RuntimeError(msg)



from csirtg_indicator import Indicator


class Client(object):

    @staticmethod
    def _kv_to_indicator(kv):
        return Indicator(**kv)

    def search(self, data):
        return self.indicators_search(data)

    def indicators_create(self, data):
        raise NotImplementedError

    def indicators_search(self, data):
        raise NotImplementedError



import pytest
from cifsdk.client.http import HTTP as Client
import responses


@pytest.fixture
def client():
    yield Client(remote='http://localhost')


@responses.activate
def test_http(client):
    assert client.remote


@responses.activate
def test_http_search(client):
    responses.add(responses.GET, 'http://localhost/indicators',
                  json=[], status=200, headers={'Content-Length': '2'})

    rv = client.indicators_search({})
    assert rv == []

    responses.add(responses.POST, 'http://localhost/indicators/bulk',
                  json=[], status=200, headers={'Content-Length': '2'})

    rv = client.indicators_search([])
    assert rv == []


@responses.activate
def test_http_create(client):
    responses.add(responses.PUT, 'http://localhost/indicators',
                  json={'data': ''}, status=200)

    rv = client.indicators_create([{}])
    assert rv == ''


@responses.activate
def test_http_delete(client):
    responses.add(responses.DELETE, 'http://localhost/indicators',
                  json={'data': ''}, status=200)

    rv = client.indicators_delete({})
    assert rv == ''
